package kr.co.vividnext.sodalive.audition

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.RoundedCornersTransformation
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.databinding.ItemAuditionListBinding
import kr.co.vividnext.sodalive.databinding.ItemAuditionListCompletedHeaderBinding
import kr.co.vividnext.sodalive.databinding.ItemAuditionListInProgressHeaderBinding
import kr.co.vividnext.sodalive.extensions.dpToPx

class AuditionListAdapter(
    private val onItemClick: (Long) -> Unit
) : ListAdapter<AuditionListAdapter.DisplayItem, RecyclerView.ViewHolder>(DiffCallback()) {
    companion object {
        private const val TYPE_IN_PROGRESS_HEADER = 0
        private const val TYPE_COMPLETED_HEADER = 1
        private const val TYPE_ITEM = 2
    }

    sealed class DisplayItem {
        data class InProgressHeader(val count: Int) : DisplayItem()
        data class CompletedHeader(val count: Int) : DisplayItem()
        data class Data(val item: GetAuditionListItem) : DisplayItem()
    }

    class DiffCallback : DiffUtil.ItemCallback<DisplayItem>() {
        override fun areItemsTheSame(oldItem: DisplayItem, newItem: DisplayItem): Boolean {
            return oldItem == newItem
        }

        override fun areContentsTheSame(oldItem: DisplayItem, newItem: DisplayItem): Boolean {
            return oldItem == newItem
        }
    }

    inner class InProgressHeaderViewHolder(
        private val binding: ItemAuditionListInProgressHeaderBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        @SuppressLint("SetTextI18n")
        fun bind(totalCount: Int) {
            binding.tvTotalCount.text = "총 ${totalCount}개"
        }
    }

    inner class CompletedHeaderViewHolder(
        private val binding: ItemAuditionListCompletedHeaderBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        @SuppressLint("SetTextI18n")
        fun bind(totalCount: Int) {
            binding.tvTotalCount.text = "총 ${totalCount}개"
        }
    }

    inner class ViewHolder(
        private val binding: ItemAuditionListBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(data: DisplayItem.Data) {
            binding.tvTitle.text = data.item.title
            binding.ivCover.load(data.item.imageUrl) {
                crossfade(true)
                placeholder(R.drawable.ic_place_holder)
                transformations(RoundedCornersTransformation(5f.dpToPx()))
            }

            if (data.item.isOff) {
                binding.blackCover.visibility = View.VISIBLE
            } else {
                binding.blackCover.visibility = View.GONE
                binding.root.setOnClickListener { onItemClick(data.item.id) }
            }
        }
    }

    override fun getItemViewType(position: Int): Int {
        return when (getItem(position)) {
            is DisplayItem.InProgressHeader -> TYPE_IN_PROGRESS_HEADER
            is DisplayItem.CompletedHeader -> TYPE_COMPLETED_HEADER
            is DisplayItem.Data -> TYPE_ITEM
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return when (viewType) {
            TYPE_IN_PROGRESS_HEADER -> {
                InProgressHeaderViewHolder(
                    ItemAuditionListInProgressHeaderBinding.inflate(
                        LayoutInflater.from(parent.context),
                        parent,
                        false
                    )
                )
            }

            TYPE_COMPLETED_HEADER -> {
                CompletedHeaderViewHolder(
                    ItemAuditionListCompletedHeaderBinding.inflate(
                        LayoutInflater.from(parent.context),
                        parent,
                        false
                    )
                )
            }

            TYPE_ITEM -> {
                ViewHolder(
                    ItemAuditionListBinding.inflate(
                        LayoutInflater.from(parent.context),
                        parent,
                        false
                    )
                )
            }

            else -> throw IllegalArgumentException("Invalid view type")
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when (val item = getItem(position)) {
            is DisplayItem.InProgressHeader -> {
                (holder as InProgressHeaderViewHolder).bind(item.count)
            }

            is DisplayItem.CompletedHeader -> (holder as CompletedHeaderViewHolder).bind(item.count)
            is DisplayItem.Data -> (holder as ViewHolder).bind(item)
        }
    }

    private val currentItems = mutableListOf<DisplayItem>()

    fun addGroupedList(newItem: GetAuditionListResponse) {
        val isOnItems = newItem.items.filter { !it.isOff }
        val isOffItems = newItem.items.filter { it.isOff }

        if (isOnItems.isNotEmpty() && currentItems.none { it is DisplayItem.InProgressHeader }) {
            currentItems.add(DisplayItem.InProgressHeader(newItem.inProgressCount))
        }
        currentItems.addAll(isOnItems.map { DisplayItem.Data(it) })

        if (isOffItems.isNotEmpty() && currentItems.none { it is DisplayItem.CompletedHeader }) {
            currentItems.add(DisplayItem.CompletedHeader(newItem.completedCount))
        }
        currentItems.addAll(isOffItems.map { DisplayItem.Data(it) })

        submitList(currentItems.toList())
    }
}
